#include <iostream>
#include <new>			//	NEW: new(nothrow)
#include <cassert>		//	NEW: assert()
#include <algorithm>      //  uninitialized_copy()
#include "myvector.h"

using namespace std;

MyVector::MyVector(const long long int n )		// Standard-, Parameterkonstruktor
    : length_( n>0 ? n : 0), val_(nullptr)
{
    //
    //                        Restrict available memory in the shell  by:
    //				ulimit
    //
    cout << "--> " << length_ << endl;
    if ( length_>0 )
    {
//        val_    = new(nothrow) double [length_];		// NEW
        val_    = new double [length_];			        // NEW
//        assert(val_ != 0);					        // NEW
        cout << "##  " << val_ << endl;
    }
}

MyVector :: MyVector(const MyVector& orig)
 : MyVector(orig.length_)
  {
      for (long long int i=0; i<length_; i++)
        {
            val_[i] = orig.val_[i];
        }
  }


MyVector :: MyVector(MyVector&& orig)
 : length_(orig.length_), val_(orig.val_)
  {
      orig.val_=nullptr;    // avoids multiple deallocation, see [Breymann, §22.2.1]
  }

//MyVector :: MyVector(const MyVector& orig)	// old Copykonstruktor
//    : length_(orig.length_), val_(0)
//{
//    if (length_>0)
//    {
//        val_ = new double [length_];
//        for (long long int i=0; i<length_; i++)
//        {
//            val_[i] = orig.val_[i];
//        }
//    }
//}

//  http://www.stroustrup.com/C++11FAQ.html#delegating-ctor
//  http://www.stroustrup.com/C++11FAQ.html#init-list
MyVector :: MyVector(const std::initializer_list<double> s) // constructor via init list
 : MyVector( static_cast<long long>(s.size()) )
{
      copy(s.begin(), s.end(), val_);
}


MyVector :: ~MyVector()				// Destruktor
{
    delete [] val_;
}


MyVector& MyVector :: operator=(const MyVector& orig)	// copy assignment
{
    if (this != &orig)
    {
        delete [] val_;
        length_ = orig.length_;
        if (length_>0)
        {
            val_ = new double [length_];
            for (long long int i=0; i<length_; i++)
            {
                val_[i] = orig.val_[i];
            }
        }
        else
        {
            val_ = 0;
        }
    }
    return *this;
}

MyVector& MyVector :: operator=(MyVector&& orig)	// move assignment
{
    if (this != &orig)
    {
        delete [] val_;
        length_ = orig.length_;
        if (length_>0)
        {
            val_ = orig.val_;
        }
        else
        {
            val_ = 0;
        }
        orig.val_=nullptr;    // avoids multiple deallocation, see [Breymann, §22.2.2]
    }
    return *this;
}

void MyVector :: resize(long long int n)
{
    if (n<0) n=0;
    double *tmp = new double [n];
    const long long int nc = min(this->size(), n);  // copy only available data
    for (long long int k=0; k<nc; ++k)
    {
        tmp[k] = val_[k];
    }
    delete [] val_;
    val_ = tmp;
    return;
}

ostream & operator<<(ostream & s, const MyVector& orig)
{
    long long int i;

    s << endl << "MyVector has " << orig.size() << " components" << endl;
    for (i=0; i<orig.size(); i++)
    {
        s << orig[i] << "  ";
    }
    s << endl;

    return s;
}

